@@ -6,6 +6,7 @@ import kaa
 from kaa.command import Commands, command, is_enable, norec, norerun
 from kaa import document
 from kaa.filetype.default import modebase
+from gappedbuf import re as gre
 
 
 class CursorCommands(Commands):
@@ -303,8 +304,7 @@ class EditCommands(Commands):
             return
 
         pos = wnd.cursor.pos
-        wnd.cursor.end()
-        nextpos = wnd.cursor.pos
+        nextpos = wnd.cursor.adjust_nextpos(pos, wnd.document.find_newline(pos))
         if pos < nextpos:
             self.delete_string(wnd, pos, nextpos)
 
@@ -360,7 +360,7 @@ class EditCommands(Commands):
         wnd.screen.selection.clear()
         indent = wnd.document.mode.on_auto_indent(wnd)
 
-    def _get_line_sel(self, wnd):
+    def get_line_sel(self, wnd):
         doc = wnd.document
         sel = wnd.screen.selection.get_range()
         if not sel:
@@ -400,7 +400,7 @@ class EditCommands(Commands):
             return
 
         doc = wnd.document
-        tol, eol = self._get_line_sel(wnd)
+        tol, eol = doc.mode.get_line_sel(wnd)
         wnd.screen.selection.set_range(tol, eol)
 
         wnd.document.undo.beginblock()
@@ -425,7 +425,6 @@ class EditCommands(Commands):
     def _dedent_line(self, wnd, pos):
         mode = wnd.document.mode
         f, t = mode.get_indent_range(pos)
-        _trace(f, t)
         if f != t:
             cols = mode.calc_cols(f, t)
         else:
@@ -441,7 +440,7 @@ class EditCommands(Commands):
             return
 
         doc = wnd.document
-        tol, eol = self._get_line_sel(wnd)
+        tol, eol = doc.mode.get_line_sel(wnd)
         wnd.screen.selection.set_range(tol, eol)
 
         wnd.document.undo.beginblock()
@@ -613,6 +612,70 @@ class EditCommands(Commands):
         kaa.app.show_dialog(doc)
 
 
+class CodeCommands(Commands):
+    @command('code.region.linecomment')
+    def linecomment(self, wnd):
+        if not wnd.screen.selection.is_selected():
+            tol = wnd.document.gettol(wnd.cursor.pos)
+            wnd.document.mode.edit_commands.insert_string(
+                    wnd, tol, wnd.document.mode.LINE_COMMENT, 
+                    update_cursor=False)
+            wnd.cursor.setpos(tol)
+            wnd.cursor.savecol()
+            return
+        else:
+            tol, eol = wnd.document.mode.get_line_sel(wnd)
+            wnd.screen.selection.set_range(tol, eol)
+    
+            wnd.document.undo.beginblock()
+            try:
+                mode = wnd.document.mode
+                while tol < wnd.screen.selection.end:
+                    wnd.document.mode.edit_commands.insert_string(
+                        wnd, tol, wnd.document.mode.LINE_COMMENT, 
+                        update_cursor=False)
+                    tol = wnd.document.geteol(tol)
+            finally:
+                wnd.document.undo.endblock()
+    
+            wnd.cursor.setpos(wnd.screen.selection.start)
+            wnd.cursor.savecol()
+
+    def _is_comment_line(self, wnd, pos):
+        reobj = gre.compile(r'[ \t]*({})'.format(
+                    gre.escape(wnd.document.mode.LINE_COMMENT)))
+        return reobj.match(wnd.document.buf, pos)
+        
+    @command('code.region.unlinecomment')
+    def uncomment(self, wnd):
+        if not wnd.screen.selection.is_selected():
+            tol = wnd.document.gettol(wnd.cursor.pos)
+            m = self._is_comment_line(wnd, tol)
+            if m:
+                f, t = m.span(1)
+                wnd.document.mode.edit_commands.delete_string(
+                    wnd, f, t)
+            return
+        else:
+            tol, eol = wnd.document.mode.get_line_sel(wnd)
+            wnd.screen.selection.set_range(tol, eol)
+    
+            wnd.document.undo.beginblock()
+            try:
+                mode = wnd.document.mode
+                while tol < wnd.screen.selection.end:
+                    m = self._is_comment_line(wnd, tol)
+                    if m:
+                        f, t = m.span(1)
+                        wnd.document.mode.edit_commands.delete_string(
+                            wnd, f, t, update_cursor=False)
+                    tol = wnd.document.geteol(tol)
+            finally:
+                wnd.document.undo.endblock()
+    
+            wnd.cursor.setpos(wnd.screen.selection.start)
+            wnd.cursor.savecol()
+
 class MacroCommands(Commands):
     @command('macro.start-record')
     @norec
@@ -681,9 +744,10 @@ class SearchCommands(Commands):
 
     def _show_searchresult(self, wnd, hit):
         if hit:
-            wnd.cursor.setpos(hit[0])
-            wnd.screen.selection.start = hit[0]
-            wnd.screen.selection.end = hit[1]
+            f, t = hit.span()
+            wnd.cursor.setpos(f)
+            wnd.screen.selection.start = f
+            wnd.screen.selection.end = t
             kaa.app.messagebar.set_message('found')
         else:
             kaa.app.messagebar.set_message('not found')
@@ -701,12 +765,12 @@ class SearchCommands(Commands):
         else:
             start =  wnd.cursor.pos
 
-        ret = wnd.document.mode.search_next(wnd, start, modebase.SearchOption.LAST_SEARCH)
+        ret = wnd.document.mode.search_next(start, modebase.SearchOption.LAST_SEARCH)
         self._show_searchresult(wnd, ret)
 
         if not ret:
             if start != 0:
-                ret = wnd.document.mode.search_next(wnd, 0, modebase.SearchOption.LAST_SEARCH)
+                ret = wnd.document.mode.search_next(0, modebase.SearchOption.LAST_SEARCH)
                 self._show_searchresult(wnd, ret)
 
     @command('search.prev')
@@ -722,12 +786,12 @@ class SearchCommands(Commands):
         else:
             start =  wnd.cursor.pos
 
-        ret = wnd.document.mode.search_prev(wnd, start, modebase.SearchOption.LAST_SEARCH)
+        ret = wnd.document.mode.search_prev(start, modebase.SearchOption.LAST_SEARCH)
         self._show_searchresult(wnd, ret)
 
         if not ret:
             if start != wnd.document.endpos():
-                ret = wnd.document.mode.search_prev(wnd, wnd.document.endpos(),
+                ret = wnd.document.mode.search_prev(wnd.document.endpos(),
                                                     modebase.SearchOption.LAST_SEARCH)
                 self._show_searchresult(wnd, ret)
 
