@@ -1,4 +1,4 @@
-import re
+import re, os
 import kaa
 from kaa.keyboard import *
 from kaa.ui.dialog import dialogmode
@@ -80,7 +80,7 @@ class GrepCommands(editorcommand.EditCommands):
                     wnd.cursor.setpos(f)
        
             filterlist.show_listdlg('Recent searches', 
-                kaa.app.config.hist_grepstr, callback)
+                kaa.app.config.hist_grepstr.get(), callback)
 
         elif dirfrom <= wnd.cursor.pos <= dirto:
             def callback(result):
@@ -90,7 +90,7 @@ class GrepCommands(editorcommand.EditCommands):
                     wnd.cursor.setpos(f)
        
             filterlist.show_listdlg('Recent directories', 
-                kaa.app.config.hist_grepdir, callback)
+                kaa.app.config.hist_grepdir.get(), callback)
 
         else:
             def callback(result):
@@ -100,7 +100,7 @@ class GrepCommands(editorcommand.EditCommands):
                     wnd.cursor.setpos(f)
                     
             filterlist.show_listdlg('Recent filenames', 
-                kaa.app.config.hist_grepfiles, callback)
+                kaa.app.config.hist_grepfiles.get(), callback)
         
 
 
@@ -131,20 +131,21 @@ class GrepDlgMode(dialogmode.DialogMode):
         else:
             config = kaa.app.config
 
-            if config.hist_grepstr:
-                GrepOption.LASTOPTION.text = config.hist_grepstr[0]
-                    
             if wnd and wnd.screen.selection.is_selected():
                 f, t = wnd.screen.selection.get_range()
                 s = wnd.document.gettext(f, t).split('\n')
                 if s:
-                    GrepOption.LASTOPTION.text = s[0].strip()
+                    s = s[0].strip()
+                    if s:
+                        GrepOption.LASTOPTION.text = s
 
-            if config.hist_grepdir:
-                GrepOption.LASTOPTION.directory = config.hist_grepdir[0]
-                    
-            if config.hist_grepfiles:
-                GrepOption.LASTOPTION.filenames = config.hist_grepfiles[0]
+            grepdir = config.hist_grepdir.get()
+            if grepdir:
+                GrepOption.LASTOPTION.directory = grepdir[0]
+
+            grepfiles = config.hist_grepfiles.get()
+            if grepfiles:
+                GrepOption.LASTOPTION.filenames = grepfiles[0]
                     
         self.option = GrepOption.LASTOPTION
         
@@ -206,6 +207,12 @@ class GrepDlgMode(dialogmode.DialogMode):
         f.append_text('caption', 'Filenames:')
         f.append_text('default', ' ')
         f.append_text('default', self.option.filenames, mark_pair='filenames')
+        f.append_text('default', '\n')
+
+        # working directory
+        f.append_text('default', '(current dir)')
+        f.append_text('default', ' ')
+        f.append_text('default', os.getcwd())
         f.append_text('default', ' ')
 
         # buttons
