@@ -7,6 +7,7 @@ from kaa.ui.msgbox import msgboxmode
 from kaa.command import command, Commands, norec, norerun
 from kaa.commands import editorcommand
 from kaa.ui.selectlist import filterlist
+from gappedbuf.sre_constants import error as gre_error
 
 SearchThemes = {
     'default':
@@ -23,23 +24,7 @@ SearchThemes = {
 
 
 class SearchCommands(editorcommand.EditCommands):
-
-    @command('edit.backspace')
-    @norec
-    @norerun
-    def backspace(self, wnd):
-        pos = wnd.cursor.pos
-        if pos > wnd.document.marks['searchtext'][0]:
-            super().backspace(wnd)
-
-    @command('edit.delete')
-    @norec
-    @norerun
-    def delete(self, wnd):
-        pos = wnd.cursor.pos
-        if pos < wnd.document.marks['searchtext'][1]:
-            super().delete(wnd)
-
+    # todo: move commands to mode class
     @command('searchdlg.search.next')
     @norec
     @norerun
@@ -69,7 +54,7 @@ class SearchCommands(editorcommand.EditCommands):
                 wnd.cursor.setpos(f)
             
         self._show_histdlg(wnd, 'Recent searches', 
-                kaa.app.config.hist_searchstr, callback)
+                kaa.app.config.hist_searchstr.get(), callback)
         
     @command('searchdlg.toggle.ignorecase')
     @norec
@@ -107,7 +92,7 @@ class SearchDlgMode(dialogmode.DialogMode):
         keybind.emacs_keys,
         keybind.macro_command_keys,
     ]
-
+    _last_searchstr = ''
     def __init__(self, target):
         super().__init__()
 
@@ -116,13 +101,16 @@ class SearchDlgMode(dialogmode.DialogMode):
         self.initialrange = target.screen.selection.get_range()
         self.lastsearch = None
         self.lasthit = None
+        self.lastmatch = None
         self.option = modebase.SearchOption.LAST_SEARCH
 
         if target and target.screen.selection.is_selected():
             f, t = target.screen.selection.get_range()
             s = target.document.gettext(f, t).split('\n')
             if s:
-                self.option.text = s[0].strip()
+                s = s[0].strip()
+                if s:
+                    self.option.text = s
 
     def close(self):
         super().close()
@@ -163,8 +151,6 @@ class SearchDlgMode(dialogmode.DialogMode):
 
         wnd.set_cursor(self.create_cursor(wnd))
         if self.option.text:
-            self.document.insert(
-                self.document.marks['searchtext'][0], self.option.text)
             wnd.screen.selection.set_range(*self.document.marks['searchtext'])
         wnd.cursor.setpos(self.document.marks['searchtext'][1])
 
@@ -210,6 +196,9 @@ class SearchDlgMode(dialogmode.DialogMode):
         self._build_buttons(f)
         self._build_options(f)
 
+        self.document.insert(
+            self.document.marks['searchtext'][0], self.option.text)
+
         self.update_option_style()
         kaa.app.messagebar.set_message("Hit alt+N/alt+P to search Next/Prev. Hit up to show history.")
 
@@ -263,16 +252,25 @@ class SearchDlgMode(dialogmode.DialogMode):
     def on_edited(self, wnd):
         self.lastsearch = None
         self.lasthit = None
-        if not self._search_next(wnd):
-            self.lastsearch = -1
-            self._search_next(wnd)
+        self.lastmatch = None
+        s = self.get_search_str()
+
+        try:
+            if s != self._last_searchstr:
+                if not self._search_next(wnd):
+                    self.lastsearch = -1
+                    self._search_next(wnd)
+            self._last_searchstr = s
+        except gre_error as e:
+            kaa.app.messagebar.set_message(str(e))
             
 
     def _show_searchresult(self, hit):
         if hit:
-            self.target.cursor.setpos(hit[0])
-            self.target.screen.selection.start = hit[0]
-            self.target.screen.selection.end = hit[1]
+            f, t = hit.span()
+            self.target.cursor.setpos(f)
+            self.target.screen.selection.start = f
+            self.target.screen.selection.end = t
             kaa.app.messagebar.set_message('found')
         else:
             self.target.screen.selection.clear()
@@ -298,16 +296,19 @@ class SearchDlgMode(dialogmode.DialogMode):
                 # resume from top
                 pos = 0
 
-            ret = self.target.document.mode.search_next(self.target, pos, self.option)
+            ret = self.target.document.mode.search_next(pos, self.option)
             self._show_searchresult(ret)
 
             self.lastsearch = pos
-            self.lasthit = ret
+            self.lasthit = ret.span() if ret else None
+            self.lastmatch = ret
 
             return ret
 
     def _save_searchstr(self):
-        kaa.app.config.hist_searchstr.add(self.get_search_str())
+        s = self.get_search_str()
+        if s:
+            kaa.app.config.hist_searchstr.add(s)
         
     def search_next(self, wnd):
         self._save_searchstr()
@@ -331,11 +332,12 @@ class SearchDlgMode(dialogmode.DialogMode):
                 # resume from end
                 pos = self.target.document.endpos()
 
-            ret = self.target.document.mode.search_prev(self.target, pos, self.option)
+            ret = self.target.document.mode.search_prev(pos, self.option)
             self._show_searchresult(ret)
 
             self.lastsearch = pos
-            self.lasthit = ret
+            self.lasthit = ret.span() if ret else None
+            self.lastmatch = ret
 
             return ret
 
@@ -351,6 +353,8 @@ class SearchDlgMode(dialogmode.DialogMode):
 
 
 class ReplaceCommands(SearchCommands):
+    # todo: move commands to mode class
+
     @command('replacedlg.field.next')
     @norec
     @norerun
@@ -373,7 +377,7 @@ class ReplaceCommands(SearchCommands):
                 wnd.cursor.setpos(f)
                 
         self._show_histdlg(wnd, 'Recent replace strings', 
-                kaa.app.config.hist_replstr, callback)
+                kaa.app.config.hist_replstr.get(), callback)
 
     @command('replacedlg.history')
     @norec
@@ -444,6 +448,12 @@ class ReplaceDlgMode(SearchDlgMode):
         self._build_buttons(f)
         self._build_options(f)
 
+        self.document.insert(
+            self.document.marks['searchtext'][0], self.option.text)
+
+        self.document.insert(
+            self.document.marks['replacetext'][0], self.option.replace_to)
+
         self.update_option_style()
         kaa.app.messagebar.set_message("Hit enter to move field. Hit up to show history.`")
 
@@ -471,11 +481,11 @@ class ReplaceDlgMode(SearchDlgMode):
     def _show_search_again(self, wnd, on_y):
         def cb(c):
             wnd.activate()
-            if c == 'y':
+            if c == 'r':
                 on_y(wnd)
 
         msgdoc = msgboxmode.MsgBoxMode.show_msgbox(
-            'Search failed. Resume again?', ['&Yes', '&Cancel'], cb)
+            'Search failed. Resume again?', ['&Resume', '&Cancel'], cb)
 
     def _save_replstr(self):
         kaa.app.config.hist_replstr.add(self.get_replace_str())
@@ -484,12 +494,10 @@ class ReplaceDlgMode(SearchDlgMode):
         if not self.get_search_str():
             return
 
+        self.option.replace_to = self.get_replace_str()
         self._save_searchstr()
         self._save_replstr()
         
-        kaa.app.config.hist_searchstr.add(self.get_search_str())
-        
-
         if self.lastsearch is None:
             self._search_next(wnd)
 
@@ -498,13 +506,30 @@ class ReplaceDlgMode(SearchDlgMode):
         else:
             self._show_search_again(wnd, self.skip_and_next)
 
-    def replace_and_next(self, wnd):
-        f, t = self.lasthit
-        newstr = self.get_replace_str()
-        self.target.document.mode.edit_commands.replace_string(
-            self.target, f, t, newstr)
+    _metachars = [
+        (r"\\", "\\"),
+        (r"\a", "\a"),
+        (r"\b", "\b"),
+        (r"\f", "\f"),
+        (r"\n", "\n"),
+        (r"\r", "\r"),
+        (r"\t", "\t"),
+    ]
 
-        self.lasthit = (f, f+len(newstr))
+    def _repl_str(self, update_cursor):
+        f, t = self.lasthit
+        if self.option.regex:
+            s = self.lastmatch.expand(self.option.replace_to)
+        else:
+            s = self.option.replace_to
+            
+        self.target.document.mode.edit_commands.replace_string(
+            self.target, f, t, s, update_cursor=update_cursor)
+
+        self.lasthit = (f, f+len(s))
+
+    def replace_and_next(self, wnd):
+        self._repl_str(update_cursor=True)
         self._search_next(wnd)
         self.search_next(wnd)
 
@@ -516,6 +541,7 @@ class ReplaceDlgMode(SearchDlgMode):
         if not self.get_search_str():
             return
 
+        self.option.replace_to = self.get_replace_str()
         self._save_searchstr()
         self._save_replstr()
 
@@ -527,19 +553,19 @@ class ReplaceDlgMode(SearchDlgMode):
             self._show_search_again(wnd, self.skip_and_prev)
 
     def replace_and_prev(self, wnd):
-        f, t = self.lasthit
-        newstr = self.get_replace_str()
-        self.target.document.mode.edit_commands.replace_string(
-            self.target, f, t, newstr)
-
-        self.lasthit = (f, f+len(newstr))
+        self._repl_str(update_cursor=True)
         self.search_prev(wnd)
 
     def skip_and_prev(self, wnd):
         self.search_prev(wnd)
 
     def replace_all(self, wnd):
+        self.option.replace_to = self.get_replace_str()
         self.option.text = self.get_search_str()
+
+        self._save_searchstr()
+        self._save_replstr()
+
         if not self.option.text:
             return
         self.lastsearch = None
@@ -548,22 +574,27 @@ class ReplaceDlgMode(SearchDlgMode):
         self.target.document.undo.beginblock()
         pos = 0
         n = 0
+        lastpos = None
         try:
             while True:
-                ret = self.target.document.mode.search_next(self.target, pos, self.option)
+                ret = self.target.document.mode.search_next(pos, self.option)
+                self.lasthit = ret.span() if ret else None
+                self.lastmatch = ret
+
                 if ret:
-                    f, t = ret
-                    self.target.document.mode.edit_commands.replace_string(
-                        self.target, f, t, newstr, update_cursor=False)
-                    self.lastsearch = (f, f+len(newstr))
-                    pos = self.lastsearch[1]
+                    self._repl_str(update_cursor=False)
+                    pos = lastpos = self.lasthit[1]
                     n += 1
                 else:
                     break
         finally:
             self.target.document.undo.endblock()
 
-        if self.lastsearch:
-            self.target.cursor.setpos(self.lastsearch[0])
+        if lastpos is not None:
+            self.target.cursor.setpos(lastpos)
 
         kaa.app.messagebar.set_message('Replaced {} time(s)'.format(n))
+
+    def on_esc_pressed(self, wnd, event):
+        self._save_replstr()
+        super().on_esc_pressed(wnd, event)
